package hr.fer.zemris.projekt.database;

/**
 * Sučelje predstavlja bazu podataka.
 * 
 * @author
 * 
 */
public interface Database {

    /**
     * Odašilje obavijest o promjeni u bazi podataka svim zainteresiranim slušačima.
     */
    public void fireDatabaseChanged();

    /**
     * Dodaje slušača zainteresiranog za promjene u bazi podataka.
     * 
     * @param l slušač zainteresiran za promjene u bazi podataka
     */
    public void addDatabaseChangeListener(DatabaseChangeListener l);

    /**
     * Uklanja slušača koj više nije zainteresiran za promjene u bazi podataka.
     * 
     * @param l slušač koji prestaje biti zainteresiran za promjene u bazi
     */
    public void removeDatabaseChangeListener(DatabaseChangeListener l);

    /**
     * Dohvaća sve podatke o sirovinama iz baze podataka. Podaci su u obliku { Integer količina,
     * Integer duljina, String tip }.
     * 
     * @return svi podaci o sirovinama iz baze podataka.
     */
    public Object[][] getStockData();

//    /**
//     * Dohvaća sve podatke o sirovinama tipa stockType iz baze podataka. 
//     * Podaci su u obliku { Integer količina, Integer duljina, String tip }.
//     * 
//     * @return svi podaci o sirovinama tipa stockType iz baze podataka.
//     */
//    public Object[][] getStockDataByType(String stockType);
    
    /**
     * Dohvaća sve nazive tipova sirovina.
     * 
     * @return
     */
    public Object[] getStockTypes();

    /**
     * Dohvaća sve podatke o tipovima sirovina. Podaci su u obliku { String naziv, Integer širina,
     * String materijal, String boja }
     * 
     * @return svi podaci o tipovima sirovina
     */
    public Object[][] getStockTypeData();

    /**
     * U bazu podataka dodaje sirovine.
     * 
     * @param size veličina tj. duljina sirovine
     * @param amount količina sirovine
     * @param stockType tip sirovine
     */
    public void insertStock(int size, int amount, String stockType);

    /**
     * Iz baze podataka briše sirovine.
     * 
     * @param size veličina tj. duljina sirovine
     * @param amount količina sirovine
     * @param stockType tip sirovine
     */
    public void removeStock(int size, int amount, String stockType);

    /**
     * U bazu podataka dodaje tip sirovine.
     * 
     * @param name naziv tipa sirovine
     * @param width širina tipa sirovine
     * @param material materijal tipa sirovine
     * @param color boja tipa sirovine
     */
    public void insertStockType(String name, int width, String material, String color);
    
    /**
     * Iz baze podataka briše tip sirovine.
     * 
     * @param name naziv tipa sirovine
     */
    public void removeStockType(String name);
    
    /**
     * Briše zapise o zalihama iz baze.
     */
    public void resetDatabase();
  
}
