package hr.fer.zemris.projekt.database.sqliteimpl;

import java.io.File;
import java.util.LinkedList;
import java.util.List;

import com.almworks.sqlite4java.SQLiteConnection;
import com.almworks.sqlite4java.SQLiteException;
import com.almworks.sqlite4java.SQLiteStatement;

import hr.fer.zemris.projekt.database.Database;
import hr.fer.zemris.projekt.database.DatabaseChangeListener;

/**
 * Razred predstavlja SQLite implementaciju baze podataka uz poboljšanja s rudimentarnim privremenim
 * spremnicima.
 * 
 * @author
 * 
 */
public class DatabaseSQLiteImpl implements Database {

    /** Slušači na promjene u bazi podataka */
    private List<DatabaseChangeListener> databaseChangeListeners;

    /** Privremeni spremnik tablice sirovina */
    private Object[][] stockDataCache;
    /** Privremeni spremnik tablice tipova sirovina */
    private Object[][] stockTypeDataCache;
    private SQLiteConnection db;
    /**
     * Konstruktor DatabaseSQLiteImpl.
     */
    public DatabaseSQLiteImpl() {
		databaseChangeListeners = new LinkedList<DatabaseChangeListener>();
		db = new SQLiteConnection(new File("./database"));
		try {
			db = db.open();
		} catch (SQLiteException e) {
			e.printStackTrace();
		}
		stockDataCache = null;
		stockTypeDataCache = null;
    }

    /**
     * See Database interface.
     */
    @Override
    public void addDatabaseChangeListener(DatabaseChangeListener l) {
    	databaseChangeListeners.add(l);
    }

    /**
     * See Database interface.
     */
    @Override
    public void removeDatabaseChangeListener(DatabaseChangeListener l) {
    	databaseChangeListeners.remove(l);
    }

    /**
     * See Database interface.
     */
    @Override
    public void fireDatabaseChanged() {
		for (DatabaseChangeListener l : databaseChangeListeners) {
		    l.databaseChanged();
		}
    }

    /**
     * See Database interface.
     */
    @Override
    public Object[][] getStockData() {
		if (stockDataCache != null) {
		    return stockDataCache;
		}
		try {
			SQLiteStatement st = db.prepare("SELECT COUNT(*) FROM sirovine");
			st.step();
			int brojRedova = st.columnInt(0);
			stockDataCache = new Object[brojRedova][3];
			st = db.prepare("SELECT * FROM sirovine");
			int i = 0;
			while(st.step()){
				stockDataCache[i][0] = st.columnInt(0);
				stockDataCache[i][1] = st.columnInt(1);
				stockDataCache[i++][2] = st.columnString(2);
			}
			st.dispose();
		} catch (SQLiteException e) {
			e.printStackTrace();
		}
		return stockDataCache;
    }
    
    /**
     * See Database interface.
     */
    @Override
    public Object[] getStockTypes() {
		Object[][] stockTypeData = getStockTypeData();
		if (stockTypeData != null){
			Object[] stockTypes = new Object[stockTypeData.length];
			for (int i = 0; i < stockTypeData.length; ++i) {
			    stockTypes[i] = stockTypeData[i][0];
			}
			return stockTypes;
		} else {
			return null;
		}
    }

    /**
     * See Database interface.
     */
    @Override
    public Object[][] getStockTypeData() {
		if (stockTypeDataCache != null) {
		    return stockTypeDataCache;
		}
	
		try {
			SQLiteStatement st = db.prepare("SELECT COUNT(*) FROM tipovi");
			st.step();
			int brojRedova = st.columnInt(0);
			stockTypeDataCache = new Object[brojRedova][4];
			st = db.prepare("SELECT * FROM tipovi");
			int i = 0;
			while(st.step()){
				stockTypeDataCache[i][0] = st.columnString(0);
				stockTypeDataCache[i][1] = st.columnInt(1);
				stockTypeDataCache[i][2] = st.columnString(2);
				stockTypeDataCache[i++][3] = st.columnString(3);
			}
			st.dispose();
		} catch (SQLiteException e) {
			e.printStackTrace();
		}
	
		return stockTypeDataCache;
    }

    /**
     * See Database interface.
     */
    @Override
    public void insertStock(int size, int amount, String stockType) {
    	if (amount == -1){
    		amount = Integer.MAX_VALUE;
    	}
    	try {
			SQLiteStatement st = db.prepare("INSERT INTO sirovine VALUES (" + size + ", " + amount + ", '" + stockType + "')");
			st.step();
			st.dispose();
		} catch (SQLiteException e) {
			e.printStackTrace();
		}
    	
		stockDataCache = null;
		fireDatabaseChanged();
    }

    /**
     * See Database interface.
     */
    @Override
    public void removeStock(int size, int amount, String stockType) {
		
    	try {
			SQLiteStatement st = db.prepare("SELECT kolicina FROM sirovine WHERE duljina=" + size + " AND tip='" + stockType + "'");
			st.step();
			int kolicina = st.columnInt(0) - amount;
			if (kolicina == 0){
				st = db.prepare("DELETE FROM sirovine WHERE duljina=" + size + " AND tip='" + stockType + "'");
			} else {
				if (st.columnInt(0) == Integer.MAX_VALUE){
					kolicina = st.columnInt(0);
				}
				st = db.prepare("UPDATE sirovine SET kolicina=" + kolicina + " WHERE duljina=" + size + " AND tip='" + stockType + "'");
			}
			st.step();
			st.dispose();
		} catch (SQLiteException e) {
			e.printStackTrace();
		}
    	
		stockDataCache = null;
		fireDatabaseChanged();
    }

    /**
     * See Database interface.
     */
    @Override
    public void insertStockType(String name, int width, String material, String color) {
		
    	try {
			SQLiteStatement st = db.prepare("INSERT INTO tipovi VALUES ('" + name + "', " + width + ", '" + material + "', '" + color + "')");
			st.step();
			st.dispose();
		} catch (SQLiteException e) {
			e.printStackTrace();
		}
    	
		stockTypeDataCache = null;
		fireDatabaseChanged();
    }
    
    /**
     * See Database interface.
     */
	@Override
	public void removeStockType(String name) {
		try {
			SQLiteStatement st = db.prepare("DELETE FROM tipovi WHERE naziv='" + name + "'");
			st.step();
			st.dispose();
		} catch (SQLiteException e) {
			e.printStackTrace();
		}
		fireDatabaseChanged();
	}
	
	/**
     * See Database interface.
     */
	@Override
	public void resetDatabase() {
		SQLiteStatement st = null;
		try {
			st = db.prepare("DELETE FROM sirovine");
			st.step();
			st = db.prepare("DELETE FROM tipovi");
			st.step();
		} catch (SQLiteException e) {
			e.printStackTrace();
		}
		if (st != null){
			st.dispose();
		}
		stockDataCache = null;
		stockTypeDataCache = null;
		fireDatabaseChanged();
	}
    
}
