package hr.fer.zemris.projekt.swing;

import javax.swing.table.DefaultTableModel;

/**
 * Razred predstavlja poboljšanu implementaciju DefaultTableModel razreda. Nudi lako podešavanje
 * mogućnosti direktnog mijenjanja vrijednosti ćelija iz grafičkog sučelja. Također nudi bolje
 * dohvaćanje razreda objekata tablice radi sortiranja i mogućnost brisanja reda unosom negativnog
 * broja ili nule u ćeliju.
 * 
 * @author
 * 
 */
public class MyTableModel extends DefaultTableModel {

    /** Serijski broj */
    private static final long serialVersionUID = 1L;

    /** True ako se iz grafičkog sučelja mogu direktno mijenjati vrijednosti ćelija, false inače */
    private boolean editable;

    /**
     * Konstruktor MyTableModel.
     * 
     * @param tableData podaci za model tablice; prosljeđuje se konstruktoru JTable
     * @param columnNames nazivi stupaca tablice; prosljeđuje se konstruktoru JTable
     * @param editable true ako se iz grafičkog sučelja mogu direktno mijenjati vrijednosti ćelija,
     *            false inače
     */
    public MyTableModel(Object[][] tableData, Object[] columnNames, boolean editable) {
	super(tableData, columnNames);

	this.editable = editable;
    }

    /**
     * Dohvaća razred stupca tablice. Služi za bolje automatsko sortiranje.
     */
    @Override
    public Class<?> getColumnClass(int column) {
	Class<?> returnValue;
	if ((getRowCount() <= 0) || (column < 0) || (column >= getColumnCount())) {
	    returnValue = Object.class;
	} else {
	    returnValue = getValueAt(0, column).getClass();
	}
	return returnValue;
    }

    /**
     * Vraća true ako je vrijednost ćelije podesiva direktno iz grafičkog sučelja, false inače.
     */
    @Override
    public boolean isCellEditable(int row, int column) {
	return editable;
    }

    /**
     * Ukoliko je vrijednost neke ćelije postavljena na negativan broj ili nulu briše taj red. Inače
     * prosljeđuje razredu JTable na obradu postavljanja ćelije.
     */
    @Override
    public void setValueAt(Object aValue, int row, int column) {
	if ((Integer) aValue <= 0) {
	    removeRow(row);
	} else {
	    super.setValueAt(aValue, row, column);
	}
    }

}
