package hr.fer.zemris.projekt.util;

import hr.fer.zemris.projekt.algorithm.Algorithm;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * Razred predstavlja konfiguraciju aplikacije. Sadrži većinu običnog teksta sučelja u konstantama
 * radi ažurnosti. Nudi metode za učitavanje dodatne konfiguracije iz vanjskih datoteka.
 * 
 * @author
 * 
 */
public final class Config {

    /** Trenutna verzija aplikacije */
    public static final String version = "1.0.0";
    /** Naslov glavnog prozora korisničkog sučelja */
    public static final String mainWindowTitle = "Projekt v" + version;
    /** Naslov prozora za pregled i dodavanje sirovina */
    public static final String stockWindowTitle = "Zalihe";
    /** Naslov prozora za pregled i dodavanje tipova sirovina */
    public static final String stockTypeWindowTitle = "Tipovi zaliha";
    /** Naslov prozora s općim informacijama o aplikaciji */
    public static final String aboutWindowTitle = "O programu";
    /** HTML/text sadržaj općih informacija o aplikaciji */
    public static final String aboutWindowText = "<html><p>Projekt v"
	    + version
	    + "</p><p></p><p>by Marin Maržić & Filip Domazet & Ante Modrić</p><p>Mentor: Prof.dr.sc. Marin Golub</p>";
    /** Sadržaj grafičkog indikatora napretka u stanu pripravnosti */
    public static final String progressBarReadyText = "Ready ...";
    /** Sadržaj grafičkog indikatora napretka u stanu rada */
    public static final String progressBarWorkingText = null;
    /** Izbornik file */
    public static final String menuFile = "Zadatak";
    /** Izbornik new */
    public static final String menuNew = "Novo";
    /** Izbornik open */
    public static final String menuOpen = "Otvori";
    /** Izbornik save */
    public static final String menuSave = "Spremi";
    /** Izbornik exit */
    public static final String menuExit = "Izađi";
    /** Izbornik edit */
    public static final String menuEdit = "Skladište";
    /** Izbornik uredi sirovine */
    public static final String menuStock = "Zalihe";
    /** Izbornik uredi tipove sirovina */
    public static final String menuStockType = "Tip zaliha";
    /** Brisanje zapisa o skladištu */
    public static final String menuReset = "Briši skladište";
    /** Izbornik informacije */
    public static final String menuInfo = "Info";
    /** Izbornik opće informacije o aplikaciji */
    public static final String menuAbout = "O programu";
    /** Kartica glavnog prozora korisničkog sučelja */
    public static final String tabMainText = "Glavni";
    /** Kartica prozora s rezultatima */
    public static final String tabResultsText = "Rezultati";
    /** Labela/okvir tekst naziva problema */
    public static final String problemNameBorder = "Naziv problema (bez razmaka)";
    /** Tekst gumba za pokretanje zadaće */
    public static final String startButtonText = "Kreni";
    /** Tekst gumba za prekidanje zadaće */
    public static final String stopButtonText = "Prekini";
    /** Labela/okvir tekst broja izvođenja pojedinog algoritma */
    public static final String numberOfRunsBorder = "Br. pokretanja";
    /** Podrazumjevana vrijednost tekst polja broja izvođenja pojedinog algoritma */
    public static final String defaultNumberOfRuns = "1";
    /** Labela/okvir tekst tablice s raspoloživim sirovinama */
    public static final String stockTableBorder = "Zalihe";
    /** Labela/okvir tekst tablice s željenim rezovima */
    public static final String itemTableBorder = "Zahtjevi za rezanjem";
    /** Labela/okvir tekst polja za unos duljine sirovine */
    public static final String stockSizeBorder = "Duljina";
    /** Labela/okvir tekst polja za unos količine sirovine */
    public static final String stockAmountBorder = "Količina(-1 za beskonačno)";
    /** Tekst gumba za dodavanje sirovine */
    public static final String stockButtonText = "Dodaj";
    /** Labela/okvir tekst polja za unos duljine željenog reza */
    public static final String itemSizeBorder = "Duljina";
    /** Labela/okvir tekst polja za unos količine željenog reza */
    public static final String itemAmountBorder = "Količina";
    /** Tekst gumba za dodavanje željenog reza */
    public static final String itemButtonText = "Dodaj izrezak";
    /** Labela/okvir padajućeg izbornika tipa sirovine */
    public static final String stockTypeFilterBorder = "Tip zaliha";
    /** Labela/okvir prostora za okvire za izbor algoritama */
    public static final String algorithmCheckBoxesBorder = "Algoritmi";
    /** Labela/okvir tekst polja mjere za dovoljno dobar rez */
    public static final String goodEnoughCutBorder = "Dobar izrezak";
    /** Labela/okvir tekst polja debljine pile tj. gubitka materijala pri jednom rezu */
    public static final String sawWidthBorder = "Debljina rezne ploče";
    /** Pretpostavljena vrijednost tekst polja debljine pile tj. gubitka materijala pri jednom rezu */
    public static final String defaultSawWidth = null;
    /** Pretpostavljena vrijednost tekst polja naziva problema */
    public static final String defaultProblemName = null;
    /** Pretpostavljena vrijednost tekst polja mjere za dovoljno dobar rez */
    public static final String defaultGoodEnoughCut = null;
    /** Pretpostavljena vrijednost obilježenosti okvira za izbor algoritama */
    public static final boolean defaultAlgorithmsSelected = false;
    /** Labela/okvir tekst polja naziva tipa sirovine */
    public static final String stockTypeNameBorder = "Naziv tipa";
    /** Labela/okvir tekst polja širine tipa sirovine */
    public static final String stockTypeWidthBorder = "Širina";
    /** Labela/okvir tekst polja materijala tipa sirovine */
    public static final String stockTypeMaterialBorder = "Materijal";
    /** Labela/okvir tekst polja boje tipa sirovine */
    public static final String stockTypeColorBorder = "Boja";
    /** Tekst gumba za dodavanje željenog tipa sirovine */
    public static final String stockTypeButtonText = "Dodaj";
    /** Labela/okvir tekst maksimalnog vremena izvođenja pojedinog algoritma */
    public static final String runTimeBorder = "Max. izvršavanje";
    /** Labela/okvir tekst maksimalnog vremena izvođenja pojedinog algoritma */
    public static final String stockFromFileButtonText = "Dodaj iz datoteke";
    /** Labela/okvir tekst maksimalnog vremena izvođenja pojedinog algoritma */
    public static final String stockTypeFromFileButtonText = "Dodaj iz datoteke";
    
    /** Struktura (mapa) koja kao ključ sadrži naziv algoritma, a kao vrijednost same algoritme */
    public static final Map<String, Algorithm> algorithms = new HashMap<String, Algorithm>();

    /**
     * Dodatna konfiguracija.
     */
    static {
    	fillAlgorithms();
    }

    /**
     * Puni strukturu (mapu) algorithms na način da je ključ naziv algoritma a vrijednost sam
     * algoritam. Podaci o algoritmima se dohvaćaju iz vanjske datoteke koja sadrži naziv, tip i
     * stazu do izvršne datoteke algoritma.
     */
    private static void fillAlgorithms() {
    	try {
			BufferedReader citac = new BufferedReader(new FileReader("config.txt"));
			String red = citac.readLine();
			while(red != null){
				String[] temp = red.split("\t");
				File f = new File(temp[2]);
				f.setExecutable(true, false);
				//System.out.println(temp[0] + "\t" + temp[1] + "\t" + temp[2]);
				algorithms.put(temp[0], new Algorithm(temp[0], temp[1], temp[2]));
				red = citac.readLine();
			}
		} catch (FileNotFoundException e1) {
			System.out.println("Popis algoritama nije pronađen");
			e1.printStackTrace();
		} catch (IOException e) {
			System.out.println("Neuspjelo čitanje iz datoteke sa popisom algoritama");
			e.printStackTrace();
		}
    }

}
